
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#include "stdafx.h"
#include "Application.h"
#include "worksheet.h"
#include <autowrap.h>

#include <sstream>
#include <string>
#include <iostream>
#include <fstream>
#include <winbase.h>


IDispatch *CApplication::s_pdOOApp    = NULL;   // (IDispatch) OpenOffice application
IDispatch *CApplication::s_pdDesktop  = NULL;   // (IDispatch) OpenOffice desktop

/*
    CApplication class provides functionality of the MS Excel Application object by using Open Office.
    It implements IApplication interface of the MS Excel.
*/
CApplication::CApplication():
    m_Workbooks (this)
    
{
    WRITE_LOG_R_W((L"CApplication::CApplication"));


    // Create Uno Service Manager
    CLSID clsid;  
    HRESULT hres = CLSIDFromProgID(L"com.sun.star.ServiceManager", &clsid); 
    if(FAILED(hres)) 
        return ;
    // Start server and get IDispatch... 
    hres = CoCreateInstance(clsid, NULL, CLSCTX_LOCAL_SERVER, IID_IDispatch, (void**) &CApplication::s_pdOOApp); 
    if(FAILED(hres))
        return ; 

    // Get Desktop and its assoc. IDispatch... 
    CComVariant result;
    CComVariant param1(L"com.sun.star.frame.Desktop");
    hres = AutoWrap(DISPATCH_METHOD, &result, CApplication::s_pdOOApp, L"createInstance", 1, param1); 
    if (FAILED(hres)) 
    {
        WRITE_LOG_R_W((L"FAILED"));
        return ;
    }
    CApplication::s_pdDesktop= result.pdispVal; 
    CApplication::s_pdDesktop->AddRef(); 
    m_Workbooks.Initialize (this);
}

CApplication::~CApplication()
{
    WRITE_LOG_R_W((L"CApplication::~CApplication()"));   
    if (CApplication::s_pdDesktop != NULL)
    {
        CApplication::s_pdDesktop->Release(); 
        CApplication::s_pdDesktop = NULL;
    }
}

STDMETHODIMP CApplication::put_UserControl(VARIANT_BOOL vbUserControl)
{
    WRITE_LOG_R_W((L"CApplication::put_UserControl(%d)", vbUserControl));
    // Do nothing   
  
    return S_OK;
}

STDMETHODIMP CApplication::put_DisplayAlerts(VARIANT_BOOL vbDisplayAlerts)
{
    WRITE_LOG_R_W((L"CApplication::put_DisplayAlerts(%d)", vbDisplayAlerts));
    // Do nothing

    return S_OK;
}

STDMETHODIMP CApplication::put_WindowState(XlWindowState State)
{
    WRITE_LOG_R_W((L"CApplication::put_WindowState(%d)", State));
    // Do nothing

    return S_OK;
}

STDMETHODIMP CApplication::put_Visible(VARIANT_BOOL vbVisible)
{
    WRITE_LOG_R_W((L"CApplication::put_Visible(%d)", vbVisible));
    // Do nothing

    return S_OK;
}

STDMETHODIMP CApplication::get_Workbooks(IWorkbooks **ppWorkbooks)
{
    WRITE_LOG_R_W((L"CApplication::get_Workbooks(...)")); 

    if (ppWorkbooks == NULL)
    {
        return E_POINTER;
    }
    *ppWorkbooks = &m_Workbooks;
    m_Workbooks.AddRef(); // returns pointer to WorkBooks
    return S_OK;
}

STDMETHODIMP CApplication::get_Sheets(ISheets **ppSheets)
{
    WRITE_LOG_R_W((L"CApplication::get_Sheets(...)"));
    return m_Workbooks.GetActiveWorkbook()->get_Sheets(ppSheets); // returns Sheets collection
}

STDMETHODIMP CApplication::get_Cells(IRange **ppRange)
{ 
    // actions of this function are obvious
    WRITE_LOG_R_W((L"CApplication::get_Cells(...)"));
    CSheets *pSheets = NULL;
    HRESULT hres = m_Workbooks.GetActiveWorkbook()->get_Sheets((ISheets **) &pSheets);
    if (FAILED (hres)) 
    {
        WRITE_LOG_R_W((L"FAILED"));
        *ppRange = NULL;
        return hres;
    }
    CWorksheet *pWorksheet;
    hres = pSheets->GetActiveSheet(&pWorksheet);
    if (FAILED(hres)) 
    {
        WRITE_LOG_R_W((L"FAILED"));
        pSheets->Release();
        *ppRange = NULL;
        return hres;
    }
    hres = pWorksheet->get_Cells (ppRange);
    if (FAILED (hres)) 
    {
        WRITE_LOG_R_W((L"FAILED"));
        *ppRange = NULL;
    }
    pSheets->Release();
    pWorksheet->Release();
    return hres;
}

STDMETHODIMP CApplication::get_Version(long Lcid, BSTR *pVersion)
{
    WRITE_LOG_R_W((L"CWorkbooks::get_Version(...)"));
    if (pVersion == NULL)
    {
        return E_POINTER;
    }

    *pVersion = ::SysAllocString(OLESTR("11.0"));

    return S_OK;
}
// NEW!!!
STDMETHODIMP CApplication::get_ActiveSheet(IDispatch** ppActiveSheet)
{
    WRITE_LOG_R_W((L"CApplication::get_ActiveSheet(...)"));
    CSheets *pSheets;
    HRESULT hres = m_Workbooks.GetActiveWorkbook()->get_Sheets((ISheets **) &pSheets);
    if (FAILED (hres)) 
    {
        WRITE_LOG_R_W((L"FAILED"));
        return hres;
    }
    CWorksheet *pWorksheet;
    hres = pSheets->GetActiveSheet(&pWorksheet);
    *ppActiveSheet = pWorksheet;
    return hres;
}


/*
    Converts cell references in a formula in the R1C1 
    reference style between relative and absolute references.
*/

STDMETHODIMP CApplication::ConvertFormula( 
    VARIANT          Formula,
    XlReferenceStyle FromReferenceStyle,
    VARIANT          ToReferenceStyle,
    VARIANT          ToAbsolute,
    VARIANT          RelativeTo,
    long             Lcid,
    VARIANT         *pResult
    )
{
    WRITE_LOG_R_W((L"CApplication::ConvertFormula(...)"));
    if (pResult == NULL)
    {
        return E_POINTER;
    }
    // Input format: 
    //  =
    //  R
    //  nRowNumber
    //  C
    //  nColNumber
    std::wstring sFormula = Formula.bstrVal;
    std::wstring sNumber  = L"";
    std::wstring::iterator iFormula     = sFormula.begin();
    std::wstring::iterator iFormulaEnd  = sFormula.end();
    std::wstring sResult = L"=$";
    std::wstring sRow    = L"";
    std::wstring sColumn = L"";

    long    lNumber;
    iFormula += 2;
    while ((iFormula != iFormulaEnd) && (*iFormula <= L'9') && (*iFormula >= L'0'))
    {
        sRow += *iFormula;
        iFormula++;
    }

    if (iFormula != iFormulaEnd) iFormula++;

    while ((iFormula != iFormulaEnd) && (*iFormula <= L'9') && (*iFormula >= L'0'))
    {
        sNumber += *iFormula;
        iFormula++;
    }

    lNumber = _wtoi(sNumber.c_str());

    

    if (lNumber < 26)
    {
        sColumn += ((WCHAR) lNumber + L'A' - 1);
    } else 
    {
        sColumn += ((WCHAR) (lNumber / 26) + L'A' - 1);
        sColumn += ((WCHAR) (lNumber % 26) + L'A' - 1);
    }
    
    sResult += (sColumn + L"$" + sRow);

    pResult->bstrVal = ::SysAllocString(sResult.c_str());
    pResult->vt      = VT_BSTR;
    return S_OK;
}


// NEW!!!
STDMETHODIMP CApplication::Quit()
{
    WRITE_LOG_R_W((L"CWorkbooks::Quit"));
    std::wstring sTmp = L"";
    m_Workbooks.GetActiveWorkbook()->Close(sTmp);
    return S_OK;
}

/*
    Returns active cell
*/
STDMETHODIMP CApplication::get_ActiveCell (IRange** ppRange)
{
    WRITE_LOG_R_W((L"CApplication::get_ActiveCell"));
    return m_Workbooks.GetActiveCell(ppRange);
}


int CApplication::GetDispatchPropertyValue(IDispatch** pIDispatch)
{
    // there are many of the Open Office functions use "com.sun.star.beans.PropertyValue",
    // using this method
    CComVariant res;
    CComVariant cvStr(L"com.sun.star.beans.PropertyValue");
    HRESULT hres = AutoWrap (DISPATCH_METHOD, &res, CApplication::s_pdOOApp, L"Bridge_GetStruct", 1, cvStr);
    if (hres == S_OK) 
    {
        *pIDispatch = res.pdispVal;
        (*pIDispatch)->AddRef();
    }
    else
    {
        *pIDispatch = NULL;
    }
    VariantInit(&res);
    return 0;
}


HRESULT CApplication::get_ActiveWorkbook(IWorkbook **ppWorkbook)
{
    WRITE_LOG_A(("get_ActiveWorkbook(...)"));
    if (NULL == ppWorkbook)
    {
        return E_FAIL;
    }

    *ppWorkbook = m_Workbooks.GetActiveWorkbook();
    if (NULL == *ppWorkbook )
    {
        WRITE_LOG_A(("get_ActiveWorkbook(...) - FAILED"));
        return E_FAIL;
    }
    return S_OK;
}

std::wstring CApplication::generateFileName()
{
    //wchar_t tcs[20];
    //SYSTEMTIME st;  // Get system date to create unique name 
    //std::wstringstream str;
    //::GetLocalTime (&st);
    //int nKf = 8;
    //if (st.wMonth < 10)
    //    nKf = 7;
    //str << L"file:///" << m_sPath << m_sName;
    //str << st.wMonth * 1000000 + st.wDay * 10000 + st.wYear; 
    //str << L'_';
    //str << st.wHour * 10000 + st.wMinute * 100 + st.wSecond;
    //str << L".xls";
    //return str.str();
    return L"";
}