
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#include "StdAfx.h"
#include "Range.h"
#include "Font.h"
#include "Application.h"

#include <autowrap.h>
#include <string>
#include <new>

/*
	CRange class represents a cell, a row, a column, a selection of cells containing 
	one or more contiguous blocks of cells, or a 3-D range. It implements IRange COM interface of the MS Excel.


*/
CRange::CRange():
	m_pdRange (NULL)
{
    WRITE_LOG_R_W((L"CRange::CRange()"));
}

CRange::~CRange()
{
    WRITE_LOG_R_W((L"CRange::~CRange()"));
	if (m_pdRange != NULL)
	{
		m_pdRange->Release();
	}
}

HRESULT CRange::Initialize(IDispatch *pThisDisp) 
{
    WRITE_LOG_R_W((L"CRange::Initialize(...)"));
	if (m_pdRange != NULL) {
		m_pdRange->Release ();
	}	
	m_pdRange = pThisDisp;
	if (m_pdRange != NULL) {
		m_pdRange->AddRef ();
		return S_OK;
	}	
	return E_POINTER;
}

HRESULT CRange::Initialize(CRange *pParentRange, const CELL_COORD &topLeft, const CELL_COORD &bottomRight)
{
    WRITE_LOG_R_W((L"CRange::Initialize(...)"));
	if (pParentRange->m_pdRange == NULL)
    {
		return E_POINTER;
    }
 	VARIANT resRange;
	VariantInit (&resRange);	
	VARIANT vLeft, vRight, vTop, vBottom;
	VariantInit(&vLeft);
	vLeft.vt = VT_I4;
	vLeft.lVal = topLeft.x - 1; // i'm sub 1 because in OO oll collectin numerates from zerro
	VariantInit(&vTop);
	vTop.vt = VT_I4;
	vTop.lVal = topLeft.y - 1;
	VariantInit(&vRight);
	vRight.vt = VT_I4;
	vRight.lVal = bottomRight.x - 1;
	VariantInit(&vBottom);
	vBottom.vt = VT_I4;
	vBottom.lVal = bottomRight.y - 1;

	HRESULT hres = AutoWrap(DISPATCH_METHOD, &resRange, pParentRange->m_pdRange, 
		L"getCellRangeByPosition", 4, vBottom, vRight, vTop, vLeft);
	if (hres != S_OK) 
    {
		m_pdRange = NULL;
		return hres;
	}
	m_pdRange = resRange.pdispVal;
    return S_OK;
}

STDMETHODIMP CRange::get__Default(VARIANT varRowIndex, VARIANT varColumnIndex, IDispatch **ppObject)
{
    WRITE_LOG_R_W((L"CRange::get__Default(...)"));

    if (ppObject == NULL)
    {
        return E_POINTER;
    }
    *ppObject = NULL;

    if (varRowIndex.vt == VT_I2)
    {
        varRowIndex.lVal = varRowIndex.iVal;
        varRowIndex.vt = VT_I4;
    }
    if (varColumnIndex.vt == VT_I2)
    {
        varColumnIndex.lVal = varColumnIndex.iVal;
        varColumnIndex.vt = VT_I4;
    }
    if (varRowIndex.vt != VT_I4 || varColumnIndex.vt != VT_I4)
    {
        return E_OUTOFRANGE;
    }

    try
    {
// Create new Range object that occupy one cell
        CRangeObject *pCell = new CRangeObject;
        CELL_COORD cellCoord(varColumnIndex.lVal, varRowIndex.lVal);
        HRESULT hr = pCell->Initialize(this, cellCoord, cellCoord);
        if (FAILED(hr))
        {
            delete pCell;
            return hr;
        }

        pCell->AddRef();
        *ppObject = pCell;
        return S_OK;
    }
    catch (const std::bad_alloc&)
    {
        return E_OUTOFMEMORY;
    }
}

STDMETHODIMP CRange::put_ColumnWidth(long nColumnWidth)
{
    WRITE_LOG_R_W((L"CRange::put_ColumnWidth(%d), nColumnWidth"));
	if (m_pdRange == NULL)
		return E_POINTER;

// It is a some different between MS excel and OO spreadsheet
// in MS excell we may get and put ColumnWidth in chars, 
// in OO spreadsheet we setiing this sizes in units 10HTMM

// for default font the appropriate ratio about 1 char  == 200(10HTMM)
	VARIANT res;
	VARIANT columns;

    HRESULT hres = S_OK;
// Get OO columns collection
	VARIANT vColWidth;
	hres = AutoWrap(DISPATCH_METHOD, &columns, m_pdRange, L"getColumns", 0);

    if (FAILED(hres))
		return hres;

// Set width
	VariantInit(&vColWidth);
	vColWidth.vt = VT_INT;
	vColWidth.intVal = nColumnWidth*210;
	hres = AutoWrap(DISPATCH_PROPERTYPUT, &res, columns.pdispVal, L"Width", 1, vColWidth);
	columns.pdispVal->Release();
    return hres;
}

STDMETHODIMP CRange::get_ColumnWidth(long *pnColumnWidth)
{
    WRITE_LOG_R_W((L"CRange::get_ColumnWidth(...)"));
// some as in put_ColumnWidth
    if (pnColumnWidth == NULL)
    {
        return E_POINTER;
    }
	if (m_pdRange == NULL)
		return E_POINTER;
	VARIANT columns;
	HRESULT hres;

	VARIANT vColWidth;
	hres = AutoWrap(DISPATCH_METHOD, &columns, m_pdRange, L"getColumns", 0);
	if (hres != S_OK) 
		return hres;
	hres = AutoWrap(DISPATCH_PROPERTYGET, &vColWidth, columns.pdispVal, L"Width", 0);
	columns.pdispVal->Release();
	*pnColumnWidth = vColWidth.intVal/200;
	return hres;
}


STDMETHODIMP CRange::get_Font(IXLFont **ppFont)
{
    WRITE_LOG_R_W((L"CRange::get_Font(...)"));

    if (ppFont == NULL)
    {
        return E_POINTER;
    }
    *ppFont = NULL;
// Create wrapper about CXLFontObject
// if some property of CXLFontObject be used, CXLFontObject be applied
// their to OO range object
    try
    {
        CXLFontObject *pFont = new CXLFontObject(this);
        HRESULT hr = pFont->Initialize(this);
        if (FAILED(hr))
        {
            WRITE_LOG_R_W((L"FAILED"));
            delete pFont;
            return hr;
        }

        pFont->AddRef();
        *ppFont = pFont;
        return S_OK;
    }
    catch (std::bad_alloc&)
    {
        return E_OUTOFMEMORY;
    }
}

STDMETHODIMP CRange::get_Value(VARIANT varRangeValueDataType, VARIANT *pvarValue)
{
    WRITE_LOG_R_W((L"CRange::get_Value(...)"));

    if (pvarValue == NULL)
        return E_POINTER;
	VARIANT resultCell;
	CComVariant parmRow(0); // XPOS 
	CComVariant parmColumn(0); // nYPos

	HRESULT hres = AutoWrap(DISPATCH_METHOD, &resultCell, m_pdRange, L"getCellByPosition", 2, parmRow, parmColumn); 
	if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FAILED"));
		return hres;
    }
	hres = AutoWrap(DISPATCH_METHOD, pvarValue, resultCell.pdispVal, L"getFormula", 0); 
	return hres;

}

STDMETHODIMP CRange::put_Value(VARIANT varRangeValueDataType, VARIANT varValue)
{
    WRITE_LOG_R_W((L"CRange::put_Value(...)"));
   
	VARIANT resultCell;	
	CComVariant  parmRow(0); // XPOS 
	CComVariant  parmColumn(0); // YPOS 

	HRESULT hres = AutoWrap(DISPATCH_METHOD, &resultCell, m_pdRange, L"getCellByPosition", 2, parmRow, parmColumn); 
	if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FAILED"));
		return hres;
    }
	VARIANT res;	
	hres = AutoWrap(DISPATCH_METHOD, &res, resultCell.pdispVal, L"setFormula", 1, varValue); 
	return hres;
}

STDMETHODIMP CRange::Select(VARIANT *pvarResult)
{
    WRITE_LOG_R_W((L"CRange::Select(...)"));
// Do nothing
    if (pvarResult == NULL)
    {
        return E_POINTER;
    }
    V_VT(pvarResult) = VT_BOOL;
    V_BOOL(pvarResult) = VARIANT_TRUE;
	
    return S_OK;
}

STDMETHODIMP_(VARIANT) CRange::ClearContents ()
{
    WRITE_LOG_R_W((L"CRange::ClearContents(...)"));
    VARIANT vRes;
    ::VariantInit(&vRes);
    vRes.vt = VT_NULL;
    return vRes;
}

// NEW!!!
STDMETHODIMP CRange::NoteText( 
    VARIANT  Text,
    VARIANT  Start,
    VARIANT  Length,
    BSTR    *pText
    )
{
    WRITE_LOG_R_W((L"CRange::NoteText(...)"));

	VARIANT      vRes;
	CComVariant  parmRow    (0); // XPOS 
	CComVariant  parmColumn (0); // YPOS 

    IDispatch *pdCell       = NULL;
    IDispatch *pdAnnotation = NULL;

	HRESULT hres = AutoWrap(DISPATCH_METHOD, &vRes, m_pdRange, L"getCellByPosition", 2, parmRow, parmColumn); 
	if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FAILED"));
		return hres;
    }
    pdCell = vRes.pdispVal;
    VariantInit(&vRes);
    hres = AutoWrap(DISPATCH_PROPERTYGET, &vRes, pdCell, L"Annotation", 0); 
    if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FAILED"));
        pdCell->Release();
        return hres;
    }
    pdAnnotation = vRes.pdispVal;
    VariantInit(&vRes);

    if (Text.vt == VT_ERROR)
    {
        hres = AutoWrap(DISPATCH_PROPERTYGET, &vRes, pdAnnotation, L"String", 0); 
        pdCell->Release();
        pdAnnotation->Release();
        
        if (hres == S_OK)
        {

            int iLength;
            if (Length.vt == VT_ERROR)
            {
                iLength = 0xffff;
            } 
            else 
            {
                iLength = Length.intVal;
            }

            int iStart;
            if (Start.vt == VT_ERROR)
            {
                iStart = 0;
            } 
            else 
            {
                iStart = Start.intVal - 1;
            }

            int iRealLen    = ::SysStringLen(vRes.bstrVal);
            wchar_t *wscResult  = new wchar_t[iRealLen];
            wchar_t *wscBeg     = wscResult;
            memcpy(wscResult, vRes.bstrVal, iRealLen * sizeof(wchar_t));
            if ((iStart < iRealLen) && (iRealLen != 0))
            {

                wscBeg += iStart;
                if (iLength < iRealLen)
                {
                    *(wscBeg + iLength) = 0;
                } else 
                {
                    *(wscBeg + iRealLen) = 0;
                }
            }
            else 
            {
                wscBeg = 0;
            }
            *pText = ::SysAllocString(wscBeg);
        }
    } 
    else 
    {
        std::wstring sNewAnnotation = L"";
        if (Start.vt != VT_ERROR)
        {
            hres = AutoWrap(DISPATCH_PROPERTYGET, &vRes, pdAnnotation, L"String", 0); 
            if (hres != S_OK)
            {
                pdCell->Release();
                pdAnnotation->Release();
                return hres;
            }
            sNewAnnotation = vRes.bstrVal;
        }
        sNewAnnotation = sNewAnnotation  + Text.bstrVal;
        CComVariant p1 = sNewAnnotation.c_str();
        hres = AutoWrap(DISPATCH_PROPERTYPUT, &vRes, pdAnnotation, L"String", 1, p1); 
        pdCell->Release();
        pdAnnotation->Release();
    }
    // TODO : Insert delete
	return hres;
}

STDMETHODIMP_(long) CRange::get_Column()
{
    WRITE_LOG_R_W((L"CRange::get_Column()"));
    
    IDispatch *pdRangeAddress = NULL;
    VARIANT vRes;
    HRESULT hres = AutoWrap(DISPATCH_METHOD, &vRes, m_pdRange, L"getRangeAddress", 0);
    if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FAILED"));
        return -1;    
    }
    pdRangeAddress = vRes.pdispVal;
    hres = AutoWrap(DISPATCH_PROPERTYGET, &vRes, pdRangeAddress, L"StartColumn", 0);
    pdRangeAddress->Release();
    if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FAILED"));
        return -1;
    }
    return vRes.intVal + 1; 
}

// NEW!!!
STDMETHODIMP_(long) CRange::get_Row()
{
    WRITE_LOG_R_W((L"CRange::get_Row()"));
    IDispatch *pdRangeAddress = NULL;
    VARIANT vRes;
    HRESULT hres = AutoWrap(DISPATCH_METHOD, &vRes, m_pdRange, L"getRangeAddress", 0);
    if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FILED"));
        return -1;    
    }
    pdRangeAddress = vRes.pdispVal;
    hres = AutoWrap(DISPATCH_PROPERTYGET, &vRes, pdRangeAddress, L"StartRow", 0);
    pdRangeAddress->Release();
    if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FILED"));
        return -1;
    }
    return vRes.intVal + 1; 
}

HRESULT CRange::GetRangeAddress(long &lLeft, long &lTop, long &lRight, long &lBottom)
{
    WRITE_LOG_R_W((L"CRange::GetRangeAddress(...)"));
    IDispatch *pdRangeAddress = NULL;
    VARIANT vRes;
    HRESULT hres = AutoWrap(DISPATCH_METHOD, &vRes, m_pdRange, L"getRangeAddress", 0);
    if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FILED"));
        return hres;    
    }
    pdRangeAddress = vRes.pdispVal;
    hres = AutoWrap(DISPATCH_PROPERTYGET, &vRes, pdRangeAddress, L"StartRow", 0);
    if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FILED"));
        pdRangeAddress->Release();
        return hres;
    }
    lLeft = vRes.intVal;

    hres = AutoWrap(DISPATCH_PROPERTYGET, &vRes, pdRangeAddress, L"StartColumn", 0);
    if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FILED"));
        pdRangeAddress->Release();
        return hres;
    }
    lTop = vRes.intVal;    

    hres = AutoWrap(DISPATCH_PROPERTYGET, &vRes, pdRangeAddress, L"EndRow", 0);
    if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FILED"));
        pdRangeAddress->Release();
        return hres;
    }
    lRight = vRes.intVal;    

    hres = AutoWrap(DISPATCH_PROPERTYGET, &vRes, pdRangeAddress, L"EndColumn", 0);
    if (hres != S_OK)
    {
        WRITE_LOG_R_W((L"FILED"));
        pdRangeAddress->Release();
        return hres;
    }
    lBottom = vRes.intVal;    
    pdRangeAddress->Release();
    return hres;
}

