
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


// Sheets.cpp

#include "StdAfx.h"
#include "Sheets.h"
#include "WorkBook.h"
#include "Worksheet.h"
#include <autowrap.h>

/*
	CSheets class provides functionality of the Sheets objects by using OpenOffice. 
	It implements ISheets COM interface.
*/
CSheets::CSheets() :
	m_pWorkbook(NULL), 
	m_pdSheets(NULL)
{
	WRITE_LOG_R_W((L"CSheets::CSheets")); 
}

CSheets::~CSheets()
{
	WRITE_LOG_R_W((L"CSheets::~CSheets")); 
	m_pdSheets->Release();
}

HRESULT CSheets::Initialize(CWorkbook *pWorkbook)
{
	WRITE_LOG_R_W((L"Sheets::Initialize"));	
	m_pWorkbook = pWorkbook;
	VARIANT resultSheets; 
	HRESULT hres = AutoWrap(DISPATCH_METHOD, &resultSheets, m_pWorkbook->m_pdDoc, L"getSheets", 0); 
	m_pdSheets =  resultSheets.pdispVal;
	m_pdSheets->AddRef();
    return hres;
}

STDMETHODIMP CSheets::get__Default(VARIANT varIndex, IDispatch **ppSheet)
{
	WRITE_LOG_R_W((L"CSheets::get_Default"));
	if ((m_pWorkbook == NULL) && (m_pdSheets == NULL)) 
    {
		return E_DATANOTINIT;
	}
	VARIANT resultSheet; 
	CWorksheet *pSheet = NULL;
	HRESULT hres; 
// Check params (I2, I4 or BSTR),  retrieve necessary sheet
// and create excel-wrapper for this sheet
    if (varIndex.vt == VT_I2) {
        varIndex.lVal = varIndex.iVal;
        varIndex.vt = VT_I4;
	}
	if (varIndex.vt == VT_I4) {		
		varIndex.iVal--;
		hres = AutoWrap (DISPATCH_METHOD, &resultSheet, m_pdSheets, L"getByIndex", 1, varIndex); 
		if (hres!=S_OK) 
			return hres;
		pSheet = new CWorksheetObject (this);		
        pSheet->Initialize(this->m_pWorkbook, resultSheet.pdispVal);
		*ppSheet = pSheet;
		(*ppSheet)->AddRef ();
		return S_OK;
	} else if (varIndex.vt == VT_BSTR) {
		hres = AutoWrap (DISPATCH_METHOD, &resultSheet, m_pdSheets, L"getByName", 1, varIndex); 
		if (hres!=S_OK) 
			return hres;
		pSheet = new CWorksheetObject (this);		
        pSheet->Initialize(this->m_pWorkbook, resultSheet.pdispVal);
		*ppSheet = pSheet;
		(*ppSheet)->AddRef ();
		return S_OK;
		
	} else {
		*ppSheet = NULL;
		return E_OUTOFRANGE;
	}
    return S_OK;
}
HRESULT CSheets::GetActiveSheet(CWorksheet **ppWorksheet) {
	WRITE_LOG_R_W((L"CSheets::GetActiveSheet"));
	VARIANT resultSheet; 
	CWorksheet *pSheet = NULL;
	VariantInit(&resultSheet); 
// Retrieve first Sheet	
// and create excel-wrapper for this sheet

    HRESULT hres = AutoWrap (DISPATCH_METHOD, &resultSheet, m_pdSheets, L"getByIndex", 1, CComVariant(0)); 
	if (hres!=S_OK) 
    {
        WRITE_LOG_R_W((L"FILED"));
		return hres;
    }
	pSheet = new CWorksheetObject (this);		
	pSheet->Initialize(this->m_pWorkbook, resultSheet.pdispVal);
	*ppWorksheet = pSheet;
	(*ppWorksheet)->AddRef ();
	return S_OK;
}

HRESULT CSheets::get_Count(long *pRhs)
{   
    WRITE_LOG_A(("CSheets::get_Count(...)"));
    *pRhs = 0;
    if (NULL == pRhs)
    {
        return E_FAIL;
    }
    HRESULT hres = S_OK;
    CComVariant res;
    hres = AutoWrap(DISPATCH_PROPERTYGET, &res, m_pdSheets, L"Count", 0);
    if (FAILED(hres))
    {
        return hres;
    }
    *pRhs = res.lVal;
    return S_OK;
}

HRESULT CSheets::Add(
    VARIANT Before, VARIANT After, VARIANT Count, VARIANT Type, long lcid, IDispatch **ppNewSheet)
{
    WRITE_LOG_A(("CSheets::Add(...)"));
    HRESULT hres = S_OK;
    CComVariant res;
    hres = AutoWrap(DISPATCH_METHOD, &res, m_pdSheets, 
            L"insertNewByName", 2, CComVariant(0), CComVariant(L"new sheet"));

    if (FAILED(hres))
    {
        WRITE_LOG_A(("CSheets::Add(...) - FAILED"));
        return hres;
    }
    
    hres = get__Default(CComVariant(1), ppNewSheet);
    return hres;
}
