
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#pragma once

#include "ExcelClone.h"
#include "InternalObject.h"
#include "Workbook.h"
#include "DispatchHook.h"
#include <vector>


class CApplication;

/*
	CWorkbooks class Represents A collection of all the Workbook objects that are currently 
	open in the Microsoft Excel application by using OpenOffice.
	It implements IWorkbooks COM interface.
*/
class ATL_NO_VTABLE CWorkbooks :
	public CComObjectRootEx<CComSingleThreadModel>,
    public CDispatchHook< IDispatchImpl< IWorkbooks, &IID_IWorkbooks, &LIBID_ExcelCloneLib, 1, 0> >
{
public:
    CWorkbooks();
    ~CWorkbooks();

    CWorkbook* GetActiveWorkbook();

    BEGIN_COM_MAP(CWorkbooks)
        COM_INTERFACE_ENTRY(IWorkbooks)
        COM_INTERFACE_ENTRY(IDispatch)
    END_COM_MAP()

public:
	/*   Creates a new workbook. 
		 The new workbook becomes the active workbook. 
		 Returns a Workbook object
	*/
    STDMETHOD(Add)(VARIANT varTemplate, IWorkbook **ppWorkbook); 

    /*
		Open existing workbook
	*/
    STDMETHOD(_Open)( 
        BSTR        Filename,
        VARIANT     UpdateLinks,
        VARIANT     ReadOnly,
        VARIANT     Format,
        VARIANT     Password,
        VARIANT     WriteResPassword,
        VARIANT     IgnoreReadOnlyRecommended,
        VARIANT     Origin,
        VARIANT     Delimiter,
        VARIANT     Editable,
        VARIANT     Notify,
        VARIANT     Converter,
        VARIANT     AddToMru,
        long        Lcid,
        IWorkbook **ppWorkbook
        );

    STDMETHOD(Open)(
        BSTR Filename, 
        VARIANT UpdateLinks, 
        VARIANT ReadOnly, 
        VARIANT Format, 
        VARIANT Password, 
        VARIANT WriteResPassword, 
        VARIANT IgnoreReadOnlyRecommended, 
        VARIANT Origin, 
        VARIANT Delimiter, 
        VARIANT Editable, 
        VARIANT Notify, 
        VARIANT Converter, 
        VARIANT AddToMru, 
        VARIANT Local, 
        VARIANT CorruptLoad, 
        long lcid, 
        IWorkbook **ppWorkbook);

    /*
		Close workbook
	*/
    STDMETHOD(Close)(long lcid);

protected:
    typedef std::vector<CWorkbookObjectPtr> CWorkbookVector;
    CWorkbookVector m_WorkbookVector;

public:
	/*
		Initialize function
	*/
	void Initialize(CApplication* pdApp);

    /*
		Return active cell in the active sheet in active workbook
	*/
    HRESULT GetActiveCell(IRange **ppRange);

protected:
	CApplication* m_pThisApp;
};

typedef CComInternalObject<CWorkbooks> CWorkbooksObject;
